#!/bin/bash
#
#    script:	convert.sh
#    author:	Pieter J. in 't Veld
#    date:	December 8, 2017, March 20, 2020.
#    purpose:	Conversion of CHARMM into EMC force field format of one or more
#		CHARMM parameter and topology files
#    notes:
#      20171208	Creation date
#      20200320 Addition of help and init
#

# settings

Version="1.1";
Date="March 20, 2020";
Script=$(basename "$0");
Frequency="false";
General="false";
Increments="true";
Redefine="";
Source="src";
Debug="false";
Show="false";
Versions="35,36";
Charmm="emc_charmm.pl";
Depth=12;

# functions

script_help() {
  printf "CHARMM to EMC force field conversion v${Version} (${Date})\n\n";
  printf "Usage:\n";
  printf "  ${Script} [-option [#]] field ...\n\n";
  printf "Options:\n";
  printf "  -help\t\tthis message\n";
  printf "  -charmm\tset script [$Charmm]\n";
  printf "  -debug\tturn on debugging mode\n";
  printf "  -define\tset alternate define file name\n";
  printf "  -depth\tset maximal scanning depth [${Depth}]\n";
  printf "  -increments\tinclude bond increments determination\n";
  printf "  -output\tselect output target; first field name when empty\n";
  printf "  -redefine\tset redefine input file\n";
  printf "  -show\t\tshow intermediate increment matrices [${Show}]\n";
  printf "  -source\tset CHARMM source directory [${Source}]\n";
  printf "  -trial\tset specific residue to convert []\n";
  printf "  -versions\tset CHARMM force field versions [${Versions}]\n";
  printf "\n";
  exit;
}


init() {
  local error=();
  local output="";
  local main="";
  local ext=""
  local name;
  local dir;

  Files=();
  while [ "$1" != "" ]; do
    case "$1" in 
      -help)	script_help;;
      -charmm)	shift; Charmm="$1";;
      -debug)	Debug="true";;
      -define)	shift; Define="$1";;
      -depth)	shift; Depth="$1";;
      -frequency) Frequency="true";;
      -general)	General="true";;
      -increments) Increments="true"; ext="_inc";;
      -output)	shift; Output="$1";;
      -redefine) shift; Redefine="$1";;
      -show)	Show="true";;
      -source)	shift; Source="$1";;
      -trial)	shift; Trial="$1";;
      -versions) shift; versions="$1";;
      -*)	script_help;;
      *)	if [ "${output}" == "" ]; then output="$1"; fi;
		if [ "${main}" == "" ]; then main="$1"; fi;
		Files+=("$1");;
    esac;
    shift;
  done;

  if [ ${#error[@]} -ne 0 ]; then
    printf "ERROR: %s\n" "${error[@]}";
    printf "\n";
    exit;
  fi;
  if [ ${#Files[@]} -eq 0 ]; then script_help; fi;
  if [ "${Output}" == "" ]; then Output="${output}${ext}"; fi;
  if [ "${Define}" == "" ]; then Define="${main}"; fi;
}


# main

{
  init $@;
 
  Commands=();

  if [ "${Depth}" != "" ]; then Commands+=("-depth=${Depth}"); fi;
  if [ "${Debug}" != "" ]; then Commands+=("-debug=${Debug}"); fi;
  if [ "${Define}" != "" ]; then Commands+=("-define=${Define}"); fi;
  if [ "${Frequency}" != "" ]; then Commands+=("-frequency=${Frequency}"); fi;
  if [ "${General}" != "" ]; then Commands+=("-general=${General}"); fi;
  if [ "${Increments}" != "" ]; then Commands+=("-increments=${Increments}"); fi;
  if [ "${Output}" != "" ]; then Commands+=("-output=${Output}"); fi;
  if [ "${Redefine}" != "" ]; then Commands+=("-redefine=${Redefine}"); fi;
  if [ "${Show}" != "" ]; then Commands+=("-show=${Show}"); fi;
  if [ "${Source}" != "" ]; then Commands+=("-source=${Source}"); fi;
  if [ "${Trial}" != "" ]; then Commands+=("-trial=${Trial}"); fi;
  if [ "${Versions}" != "" ]; then Commands+=("-versions=${Versions}"); fi;

  echo "$Charmm" "${Commands[@]}" "${Files[@]}";

  "$Charmm" "${Commands[@]}" "${Files[@]}";
  
#  emc_${HOST} field.emc "${Output}"
}

