#!/bin/bash

search() {
  #echo "$1" "$2" "$3" 1>&2;
  set -f; grep -n -w -i "$1" $(find "$3" -name "$2" -a ! -type d); set +f;
}


exists() {
  local residue=$(perl -e 'print((split(":",@ARGV[0]))[0])' "$1");
  local lines="$(search "$residue" '*' src/. | egrep -i ":[resi|pres]")";

  if [ "${lines}" == "" ]; then
    lines="$(search "$residue" '*.define' . | egrep -i ":[resi|pres]")";
  fi;
  echo ${lines}; 
}


init() {
  local trial;
  local debug=false;
  local error=true;
  local name="";
  local open=false;
  local output="";
  local show=false;
  local version=35,36;

  local all=false;
  local carb=false;
  local carbs=(
      carb carb_model);
  local cgenff=false;
  local cgenffs=(
      cgenff basf);
  local ether=false;
  local ethers=(
      ethers);
  local lipid=false;
  local lipids=(
      lipid lipid_bacterial lipid_cardiolipin lipid_cholesterol
      lipid_detergent lipid_model lipid_yeast);
  local lipid=false;
  local na=false;
  local nas=(
      na na_model na_modifications #na_nad_ppi na_reactive_rna
  );
  local prot=false;
  local prots=(
      prot prot_aldehydes prot_arg0 prot_fluoro_alkanes prot_heme
      prot_model prot_modify_res prot_pyridines prot_retinol);
  local prots=(
      prot);
  local water_ion=false;
  local water_ions=(
      water_ions);
  local fields=();
  local names=();

  Commands=(-general);
  while [ "$1" != "" ]; do
    case "$1" in
      -all)	all=true;;
      -carb)	carb=true;;
      -cgenff)	cgenff=true;;
      -debug)	debug=true;;
      -error)	error=true;;
      -ether)	ether=true;;
      -lipid)	lipid=true;;
      -na)	na=true;;
      -noerror)	error=false;;
      -open)	open=true;;
      -output)	shift; output="$1";;
      -prot)	prot=true;;
      -show)	show=true;;
      -trial)	shift; Trial="$1";;
      -version)	shift; version="$1";;
      -water_ions) water_ion=true;;
    esac;
    shift;
  done;

  if [ "${all}" == true ]; then 
    carb=true; ether=true; lipid=true; na=true; prot=true; water_ions=true;
  fi;
  if [ "${prot}" == true ]; then
    if [ "${name}" != "" ]; then name="${name}_prot"; else name=prot; fi;
    fields+=(${prots[@]});
    names+=(prot);
  fi;
  if [ "${water_ion}" == true ]; then
    if [ "${name}" != "" ]; then name="${name}_water_ions"; else name=water_ions; fi;
    fields+=(${water_ions[@]});
    names+=(water_ions);
  fi;
  if [ "${cgenff}" == true ]; then
    if [ "${name}" != "" ]; then name="${name}_cgenff"; else name=cgenff; fi;
    fields+=(${cgenffs[@]});
    names+=(cgenff);
  fi;
  if [ "${carb}" == true ]; then
    if [ "${name}" != "" ]; then name="${name}_carb"; else name=carb; fi;
    fields+=(${carbs[@]});
    names+=(carb);
  fi;
  if [ "${ether}" == true ]; then 
    if [ "${name}" != "" ]; then name="${name}_ether"; else name=ether; fi;
    fields+=(${ethers[@]});
    names+=(ether);
  fi;
  if [ "${lipid}" == true ]; then
    if [ "${name}" != "" ]; then name="${name}_lipid"; else name=lipid; fi;
    fields+=(${lipids[@]});
    names+=(lipid);
  fi;
  if [ "${na}" == true ]; then
    if [ "${name}" != "" ]; then name="${name}_na"; else name=; fi;
    fields+=(${nas[@]});
    names+=(na);
  fi;
  if [ "${all}" == true ]; then name=general; fi;

  if [ "${carb}" == true -a "${lipid}" == true ]; then
    fields+=(carb_glycolipid lipid_glycolipid lipid_inositol lipid_llo);
  fi;
  if [ "${carb}" == true -a "${prot}" == true ]; then
    fields+=(carb_glycopeptide);
  fi;
  if [ "${lipid}" == true -a "${prot}" == true ]; then
    fields+=(lipid_prot lipid_sphingo);
  fi;

  if [ ${#fields[@]} == 0 ]; then
    printf "Error: No field set\n\n";
    exit -1;
  fi;

  Commands=();
  Commands+=(-versions ${version});
  if [ "${debug}" == true ]; then Commands+=(-debug); fi;
  if [ "${output}" == "" ]; then output=${name}; fi;
  if [ "${output}" != "" ]; then Commands+=(-output ${output}); fi;
  if [ ${#names[@]} != 1 -a -e general.define ]; then
    Commands+=(-general -redefine general.define);
  fi;
  if [ "${Trial}" != "" ]; then
    if [ "${show}" == true ]; then Commands+=(-show); fi;
    Commands+=(-debug -trial ${Trial});
    Trial=$(perl -e 'print(uc(@ARGV[0]));' ${Trial});
  fi;
  Commands+=(${fields[@]});
  Error=${error};
  Open=${open};
}


# main

{
  init $@;
  if [ "${Trial}" == "" ]; then 
    ../convert.sh ${Commands[@]};
  else
    if [ "$(exists ${Trial})" == "" -a "${Error}" == true ]; then
      printf "Error: trial ${Trial} not found\n\n";
      exit;
    fi;
    ../convert.sh ${Commands[@]} >& trial/${Trial}.out;
    if [ "${Open}" == true ]; then
      gvim trial/${Trial}.out;
    fi;
  fi;
}

