#!/bin/bash

search() {
  set -f; grep -n -w -i "$1" $(find "$3" -name "$2" -a ! -type d); set +f;
}


help() {
  printf "Usage: $0 [-open] [-field field] RESI\n\n";
  exit;
}


init() {
  local resi=();
  local open=false;

  while [ "$1" != "" ]; do
    case "$1" in
      -open)	open=true;;
      -field)	shift; field="$1";;
      -*)	help;;
      *)	resi+=("$1");;
    esac;
    shift;
  done;

  if [ ${#resi[@]} == 0 ]; then help; fi;

  Residue=(${resi[@]});
  Open=$open;
}


open() {
  local command=($(
    perl -e '@a=split(":", @ARGV[0]); print("@a[0] +@a[1]");' "$1"));
  gvim ${command[@]};
}


# main

{
  init $@;
  for resi in ${Residue[@]}; do
    lines="$(search "$resi" "*" src/. | egrep -i ':[resi|pres]')";
    if [ "${lines}" == "" ]; then
      lines="$(search "$resi" "*.define" . | egrep -i ':[resi|pres]')";
    fi;
    printf "$lines\n";
    if [ "$Open" == true ]; then
      for line in $( \
	perl -e '
	    $field = @ARGV[0];
	    foreach(split("\n", @ARGV[1])) {
	      my @a = split(":");
	      print(join(":", @a[0,1])." ") if (@a[0] =~ m/$field/g);
	    }' "${field}" "${lines}"); do
	open "$line";
      done;
    fi;
  done;
}

