#!/usr/bin/env perl
#
#  script:	compare.pl
#  author:	Pieter J. in 't Veld
#  date:	August 15, 2023
#  purpose:	Compare cgenff_BASF parameter with toppar_basf parameter
#
#  notes:
#    20230815	No missing parameter were found in 
use Data::Dumper;

sub number_q {					# <= number
  return @_[0] =~ m/^([+-]?)(?=\d|\.\d)\d*(\.\d*)?([Ee]([+-]?\d+))?$/ ? 1 : 0;
}


sub dumper {
  my $fmt = ref(@_[0]) eq "" ? shift(@_) : "";

  $Data::Dumper::Sortkeys = 1;
  print($fmt, Dumper(@_[0]));
}


sub arrange_compare {
  my ($a, $b) = @_[0,1];
  return 0 if ($a eq "*");
  return 1 if ($b eq "*");
  $a = substr($a,1) if (substr($a,0,1) =~ m/\+|\-]/);
  $b = substr($b,1) if (substr($b,0,1) =~ m/\+|\-]/);
  return $a lt $b ? 1 : 0;
}


sub arrange {
  return @_ if (scalar(@_)<2);
  return @_ if (arrange_compare(@_[0, -1]));
  if (@_[0] eq @_[-1]) {
    return @_ if (scalar(@_)<4);
    return @_ if (arrange_compare(@_[1, 2]));
    return @_[3,2,1,0];
  }
  return @_[1,0] if (scalar(@_) == 2);
  return @_[2,1,0] if (scalar(@_) == 3);
  return @_[3,2,1,0];
}


sub differences {
  my $dest = shift(@_);
  my $src = shift(@_);
  my $diff = shift(@_);

  foreach (keys(%{$src})) {
    my $target = $_;

    foreach (keys(%{$src->{$target}})) {
      next if (defined($dest->{$target}->{$_}));
      $diff->{$target}->{$_} = $src->{$target}->{$_};
    }
  }
  return $diff;
}


{
  if (scalar(@ARGV)!=2) {
    print("usage: $0 file1 file2\n\n");
    exit(-1);
  }

  my $i = 0;
  my $stream;
  my @array;
  my $diff = {};

  foreach (@ARGV) {
    my $name = $_;
    my $input = {};

    print("Info: checking $name...\n");
    open($stream, "<", $_);
    foreach (<$stream>) {
      my @key;
      my @values;
      my $flag;
      
      next if (substr($_,0,1) eq "!");
      foreach (split(" ")) {
	next if ($_ =~ m/!/g);
	if (number_q($_)) {
	  push(@values, $_);
	  $flag = 1;
	} else {
	  last if ($flag);
	  push(@key, $_);
	}
      }
      my $n = scalar(@key);
      my $target = $n==2 ? "bond" : $n==3 ? "angle" : "torsion";
      my $index = join("\t", arrange(@key));

      $input->{$target}->{$index} = join("\t", @values);
    }
    close($stream);
    push(@array, $input);
    ++$i;
  }
  differences(@array[0], @array[1], $diff);
  differences(@array[1], @array[0], $diff);

  foreach(sort(keys(%{$diff}))) {
    my $target = $_;

    print("ITEM\t", uc($target), "\n\n");
    foreach (sort(keys(%{$diff->{$target}}))) {
      print($_, "\t", $diff->{$target}->{$_}, "\n");
    }
    print("\nITEM\tEND\t# ", uc($target), "\n\n");
  }
}

