#!/usr/bin/env perl

# packages

use File::Basename;

use lib $ENV{EMC_ROOT}."/scripts/modules";
use EMC;

use strict;


# functions

sub value_q {
  return @_[0] =~ m/^([+-]?)(?=\d|\.\d)\d*(\.\d*)?([Ee]([+-]?\d+))?$/ ? 1 : 0;
}


sub trim {
  my $s = shift(@_);
  $s =~ s/^\s+|\s+$//g;
  return $s;
}


# main

{
  my $header = {
    BONDS 	=> "END

read para card flex append
BONDS
!
!V(bond) = Kb(b - b0)**2
!
!Kb: kcal/mole/A**2
!b0: A
!
!atom type Kb          b0
!",
    ANGLES	=> "
ANGLES
!
!V(angle) = Ktheta(Theta - Theta0)**2
!
!V(Urey-Bradley) = Kub(S - S0)**2
!
!Ktheta: kcal/mole/rad**2
!Theta0: degrees
!Kub: kcal/mole/A**2 (Urey-Bradley)
!S0: A
!
!atom types     Ktheta    Theta0   Kub     S0
!",
    DIHEDRALS	=> "
DIHEDRALS
!
!V(dihedral) = Kchi(1 + cos(n(chi) - delta))
!
!Kchi: kcal/mole
!n: multiplicity
!delta: degrees
!
!atom types             Kchi    n   delta
!",
    IMPROPERS	=> "
IMPROPERS
!
!V(improper) = Kpsi(psi - psi0)**2
!
!Kpsi: kcal/mole/rad**2
!psi0: degrees
!note that the second column of numbers (0) is ignored
!
!atom types           Kpsi                   psi0
!",
    NONBONDED	=> "
NONBONDED nbxmod  5 atom cdiel fshift vatom vdistance vfswitch -
cutnb 14.0 ctofnb 12.0 ctonnb 10.0 eps 1.0 e14fac 1.0 wmin 1.5 
!
!V(Lennard-Jones) = Eps,i,j[(Rmin,i,j/ri,j)**12 - 2(Rmin,i,j/ri,j)**6]
!
!epsilon: kcal/mole, Eps,i,j = sqrt(eps,i * eps,j)
!Rmin/2: A, Rmin,i,j = Rmin/2,i + Rmin/2,j
!
!atom  ignored    epsilon      Rmin/2   ignored   eps,1-4       Rmin/2,1-4
!"};
  my $field;
  my $modes = [
    "MASS", "RESI", "PRES", "NONBONDED", "BONDS", "ANGLES", "DIHEDRALS",
    "IMPROPERS"];
  my $index = 0;
  my $line = 0;
  my $read = 0;
  my $last;
  my $mode;
  my $key;
  my $ptr;

  foreach (@{$modes}) { $mode->{$_} = $index++; }
  print("* Topology and Parameter Stream File for BASF additions
*

!Parent files that have to be read prior to streaming this file

read rtf card append
* Topology for heme and related compounds and patches
*
36  1

");
  foreach(<>) {
    chomp();
    $_ = trim($_);
    my @arg = split(" ");

    ++$line;
    $_ = "" if (substr($_,0,1) eq "!");
    next if ($_ eq "" ? $last eq "" : 0);
    $last = $_;
    if (defined($mode->{@arg[0]})) {
      $index = $mode->{@arg[0]};
      $key = @arg[0];
      $read = 0;
    } elsif (lc(@arg[0]) eq "end") {
      $read = 0;
    }
    if (!$read) {
      $read = 1 if ($_ =~ m/BASF/);
      next if (!$read);
    }
    $field->{$key} = [] if (!defined($field->{$key}));
    push(@{$field->{$key}}, $_);
    $read = $mode->{$key}>2 ? 0 : 1;
  }
  foreach (@{$modes}) {
    if (defined($field->{$_})) {
      print($header->{$_}, "\n") if ($mode->{$_}>2);
      foreach ($mode->{$_}>2 ? sort(@{$field->{$_}}) : @{$field->{$_}}) {
	$index = index($_, "!");
	if ($index>0) {
	  $last = "! ".substr($_, $index+(substr($_,$index+1,1) eq " " ? 2:1));
	} else {
	  $last = "";
	}
	my @a = split(" ", $index>0 ? substr($_,0,$index) : $_);
	@a = (@a, "", "", "") if (@a[0] eq "GROUP" && $index>0);
	print(join(" ", map({sprintf(value_q($_) ? "%7s" : "%-7s", $_)} @a), $last), "\n");
      }
    }
  }
  print("\nEND\n\nreturn\n");
}

